# qakit - tools for UEQA engineers
# Copyright (C) 2014 Thomi Richards
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


"Format functions for output."
from collections import defaultdict
import prettytable
import termcolor
import datetime
import dateutil.parser

_title_opts = {
    'color': 'red',
    'attrs': ['bold'],
}


_card_title_opts = {
    # 'color': 'white'
}

_card_lane_opts = {
    # 'color': 'grey'
}

def print_title(text):
    print(termcolor.colored(text, **_title_opts))


def print_card_table(cards, fields=('Name', 'Lane')):
    tbl = prettytable.PrettyTable(["Name", "Lane"])
    tbl.align["Name"] = 'l'

    for card in cards:
        tbl.add_row([
            termcolor.colored(card.name, **_card_title_opts),
            termcolor.colored(card.lane, **_card_lane_opts),
            ])
    print(tbl)


def print_actions_table(actions):
    period = _get_one_fortnight_ago()
    actions = [a for a in actions if period.contains(a.when)]

    # populate 'fields' and 'people'
    fields = ['Name']
    people = {}
    for a in actions:
        if a.what not in fields:
            fields.append(a.what)
        if a.who not in people:
            people[a.who] = {'updateCard':0, 'commentCard':0}
        if a.what not in people[a.who]:
            people[a.who][a.what] = 0
        people[a.who][a.what] += 1
    fields.sort()
    # put these fields first
    fields.remove('updateCard')
    fields.insert(1, 'updateCard')
    fields.remove('commentCard')
    fields.insert(2, 'commentCard')

    foo = [(-v['updateCard'],-v['commentCard'],k,v) for k,v in people.items()]
    foo.sort()
    people = [(x[-2],x[-1]) for x in foo]

    tbl = prettytable.PrettyTable(fields)
    tbl.align["Name"] = 'l'

    for person, actions in people:
        row = []
        row.append(termcolor.colored(person, **_card_title_opts))
        for f in fields[1:]:
            amount = 0
            if f in actions:
                amount = actions[f]
            row.append(termcolor.colored(str(amount), **_card_lane_opts))
        tbl.add_row(row)

    print(tbl)


def print_last_day_stats(cards):
    print_title("Last 24 Hours:")
    period = _get_24_hours_ago()
    _print_completed_table(cards, period)
    print("")


def print_last_week_stats(cards):
    print_title("Last Week:")
    period = _get_one_week_ago()
    _print_completed_table(cards, period)
    print("")

class StatisticGroup(object):

    """Statistics are processed in groups.

    The main reason for this is to have a single interface capable of yielding
    more than one data point (or, more to the point, a _variable number_ of
    data points).

    This is the interface for a group of statistics. There are a couple of
    concrete implementations in this file.

    """

    def get_rows(self, *periods):
        """Return a generator that yields lists of data lines.

        Each line must contain the name of the data point plus values for each
        period in the period list passed in.

        """
        raise NotImplemented("Sub-classes must implement this method.")

    def process_card(self, card, period):
        """Process card `card`, which is in period `period`.

        This simply calls 'process_card' on every statistic object in this
        group.
        """
        raise NotImplemented("Sub-classes must implement this method.")


class GenericStatisticGroup(StatisticGroup):

    """A generic group that contains other statistic objects."""

    def __init__(self, stats):
        """Create a new StatisticGroup object. Pass in an iterable of statistic
        objects.

        """
        self.statistics = stats

    def get_rows(self, *periods):
        """Return a generator that yields lists of data lines.

        Each line will contain the name of the data point plus values for each
        period in the period list passed in.

        """
        for s in self.statistics:
            yield [s.name] + [s.result_for_period(p) for p in periods]

    def process_card(self, card, period):
        """Process card `card`, which is in period `period`.

        This simply calls 'process_card' on every statistic object in this
        group.
        """
        for s in self.statistics:
            s.process_card(card, period)


class SiloTesterStatisticGroup(StatisticGroup):

    """A statistic group that keep strack of how many silos each tester has
    completed. This group yields data points equal to the number of silo testers
    in the cards processed.

    """
    _statistics = {}

    def process_card(self, card, period):
        if card.assigned_to not in self._statistics:
            self._statistics[card.assigned_to] = TesterCountStatistic(card.assigned_to)
        self._statistics[card.assigned_to].process_card(card, period)

    def get_rows(self, *periods):
        """Return a generator that yields lists of data lines.

        Each line will contain the name of the data point plus values for each
        period in the period list passed in.

        """
        for s in self._statistics:
            yield [s] + [self._statistics[s].result_for_period(p) for p in periods]


class Statistic(object):

    """An interface for a single statisitc."""

    name = ""

    def process_card(self, card, period):
        """Process card `card`, which is in period `period`.

        Implementations should store whatever data they want with `period` as
        the key, since the caller will later use the same period to retrieve
        any results.

        Implementations may assume that 'card' is within 'period'.
        """
        raise NotImplemented("Sub-classes must implement this method.")

    def result_for_period(self, period):
        """Return a string representing the results for this period.

        If not results are stored for this period, return "NA".
        """
        raise NotImplemented("Sub-classes must implement this method.")


class PassRateStatistic(Statistic):
    """Calculate the pass rate in a given period."""

    def __init__(self):
        self._data = defaultdict(lambda: dict(total=0.0, passed=0.0))
        self.name = "Pass Rate"

    def process_card(self, card, period):
        self._data[period]['total'] += 1.0
        if card.final_lane.lower() == 'passed':
            self._data[period]['passed'] += 1.0

    def result_for_period(self, period):
        if period not in self._data:
            return "NA"
        if self._data[period]['total'] <= 0:
            return "NA"
        return "%.0f%%" % (
            self._data[period]['passed'] / self._data[period]['total'] * 100
        )


class TesterCountStatistic(Statistic):
    """Calculate the number of cards completed, as well as a pass rate."""

    def __init__(self, name):
        self._data = defaultdict(lambda: dict(total=0.0, passed=0.0))
        self.name = name

    def process_card(self, card, period):
        self._data[period]['total'] += 1.0
        if card.final_lane.lower() == 'passed':
            self._data[period]['passed'] += 1.0

    def result_for_period(self, period):
        if period not in self._data:
            return "NA"
        percentage = ''
        if self._data[period]['total'] > 0:
            percentage = "(%.0f%%)" % (
                self._data[period]['passed'] / self._data[period]['total'] * 100
            )
        return "%d total, %d passed %s" % (
            self._data[period]['total'],
            self._data[period]['passed'],
            percentage
            )


class Period(object):
    """A span of time between two datetime objects."""

    def __init__(self, start, end):
        if start > end:
            raise ValueError("Start date must be before or equal to end date")
        self.start = start
        self.end = end

    def contains(self, datetime):
        return self.start <= datetime <= self.end

    def __hash__(self):
        return hash(self.start) | hash(self.end)


def print_meta_stats(cards):
    # TODO: Define periods, period_names globally and passs them in. Use these
    # same valies for the _print_completed_table calls as well.
    periods = [
        _get_24_hours_ago(),
        _get_one_week_ago(),
    ]
    period_names = [
        "Last Day",
        "Last Week",
    ]

    stat_groups = [
        GenericStatisticGroup([PassRateStatistic()]),
        SiloTesterStatisticGroup()
    ]

    for card in cards:
        for period in periods:
            if period.contains(card.date):
                for group in stat_groups:
                    group.process_card(card, period)

    tbl = prettytable.PrettyTable(["Category"] + period_names)
    tbl.add_row(["Period Starting"] + [p.start.strftime("%c") for p in periods])
    for group in stat_groups:
        for row in group.get_rows(*periods):
            tbl.add_row(row)
    print(tbl)


def _print_completed_table(cards, period):
    tbl = prettytable.PrettyTable(["Date", "Tester", "Status", "Name"])
    tbl.align["Name"] = "l"
    tbl.align["Tester"] = "l"
    for card in cards:
        if not period.contains(card.date):
            continue
        tbl.add_row([
            card.date.strftime("%c"),
            card.assigned_to,
            _get_coloured_status_text(card.final_lane),
            card.name
            ])
    print(tbl)


def _get_coloured_status_text(status):
    if status.lower() == 'passed':
        return termcolor.colored(status, color="green")
    else:
        return termcolor.colored(status, color="red")


def _get_24_hours_ago():
    """Get a timezone-aware datetime object that represents 24 hours ago."""
    start = datetime.datetime.utcnow().replace(tzinfo=dateutil.tz.tzutc()) \
        - datetime.timedelta(days=1)
    end = datetime.datetime.utcnow().replace(tzinfo=dateutil.tz.tzutc())
    return Period(start, end)


def _get_one_week_ago():
    """Get a timezone-aware datetime object that represents 7 days ago."""
    start = datetime.datetime.utcnow().replace(tzinfo=dateutil.tz.tzutc()) \
        - datetime.timedelta(days=7)
    end = datetime.datetime.utcnow().replace(tzinfo=dateutil.tz.tzutc())
    return Period(start, end)


def _get_one_fortnight_ago():
    """Get a timezone-aware datetime object that represents 14 days ago."""
    start = datetime.datetime.utcnow().replace(tzinfo=dateutil.tz.tzutc()) \
        - datetime.timedelta(days=14)
    end = datetime.datetime.utcnow().replace(tzinfo=dateutil.tz.tzutc())
    return Period(start, end)
