#!/usr/bin/python3
# PractiSubunit
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import configparser
import os
import sys

from qakit.practitest import practisubunit
from qakit.practitest import subunit_results

DOMAIN_FILTER_IDS = {'Sanity': '55736',
                     'Web': '55419'}


def _parse_arguments():
    """Parse command-line args, returning an argparse dict."""
    parser = argparse.ArgumentParser(
        'Report subunit suite results to Practitest.')
    parser.add_argument('filepath', help='subunit result filepath')
    parser.add_argument('--testset-number', '-t',
                        type=int,
                        help='PractiTest TestSet #')
    parser.add_argument('--testset-name', '-n',
                        help='The name of the test set to create')
    parser.add_argument('--manual', '-m',
                        help='Add the manual tests for the specified domain '
                             'to the test set as well.')
    parser.add_argument('--device', '-d',
                        help='The name of the device the tests were run on')
    parser.add_argument('--release', '-r',
                        help='The release of Ubuntu the tests were run on')
    parser.add_argument('--build', '-b',
                        help='The build ID of the image the tests were run on')
    return parser.parse_args()


def _confirm_name_of_testset(practitest_testset_number,
                             practitest_session):
    """Double-check the name and assigned tester of the numbered TestSet.

    jibel asks for this--it'll be hard to undo result entries in the
    wrong TestSet, e.g.

    """
    testset = practitest_session.get_testset(practitest_testset_number)
    confirmation = input(
        ("TestSet {}: '{}', assigned to {}--"
         "upload test results (Y/n)? ").format(
             practitest_testset_number,
             testset['name'],
             testset['assigned_to']['value']))
    return confirmation != 'n'


def _read_config_for_var(var_name, config_file):
    """Retrieve the config var, defaulting to an env var when set."""
    try:
        return os.environ[var_name]
    except KeyError:
        try:
            return config_file['DEFAULT'][var_name]
        except KeyError:
            raise KeyError(
                ('{} not specified in config or '
                 'by environment variable.').format(var_name))


def _read_config(config_filepath):
    """Parse the config at the given filepath, returning a config dict."""
    config_file = configparser.ConfigParser()
    config_file.read(config_filepath)
    config = {}
    for var_name in ('PRACTITEST_PROJECT_ID',
                     'PRACTITEST_API_KEY',
                     'PRACTITEST_API_SECRET_KEY'):
        new_var_name = var_name.lower().replace('practitest_', '')
        config[new_var_name] = _read_config_for_var(var_name,
                                                    config_file)
    return config


def _create_testset(testset_name, device, release, build, session):
    return session.create_testset(testset_name, 'medium',
                                  device, 'Regression',
                                  release, build)


def _create_instances(results, testset, session, manual=None):
    # First get all the automated tests in PractiTest
    # 57803 is the ID of the 'Automated' filter
    instances = []
    if manual:
        tests = session.get_tests(filter_id=DOMAIN_FILTER_IDS[manual])
        manual_ids = [test['system_id'] for test in tests]
        instances.extend(manual_ids)
    else:
        tests = session.get_tests(filter_id='57803')
        external_ids = [test[session.pt_field('external_id')]['value']
                        for test in tests]
        for test in results:
            try:
                instances.append(
                    tests[external_ids.index(test['id'])]['system_id']
                )
            except ValueError:
                continue

    session.create_instances(testset['id'], instances)


def main():
    args = _parse_arguments()
    config = _read_config(
        os.path.join(os.path.dirname(__name__),
                     'config.ini'))
    practisubunit_session = practisubunit.PractiSubunitSession(**config)
    results = subunit_results.parse(args.filepath)
    testset_number = args.testset_number
    if args.testset_number:
        _confirm_name_of_testset(testset_number, practisubunit_session)
        testset_system_id = practisubunit_session.get_testset(
            args.testset_number)['system_id']
    else:
        testset = _create_testset(args.testset_name, args.device,
                                  args.release, args.build,
                                  practisubunit_session)
        if testset:
            testset_number = testset['id']
            testset_system_id = testset['system_id']
            _create_instances(results, testset, practisubunit_session,
                              args.manual)

    practisubunit_results = practisubunit.PractiSubunitResultSet(
        practisubunit_session,
        testset_number,
        results)
    practisubunit_results.report_to_practitest()
    print(
        "Reported results to "
        "https://prod.practitest.com/p/{}/sets/{}/edit".format(
            config['project_id'], testset_system_id
        )
    )


if __name__ == '__main__':
    sys.exit(main())
