# qakit - tools for UEQA engineers
# Copyright (C) 2014-2015 Canonical
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

import dateutil.parser

import qakit.trello as trello
from qakit import format


class Card(object):
    name = u''
    lane = u''
    assigned_to = u''

    def __init__(self, name=u'', lane=u'', assigned_to=u''):
        self.name = name
        self.lane = lane
        self.assigned_to = assigned_to


class CompletedCard(object):
    final_lane = u''
    date = None
    assigned_to = u''
    name = u''

    def __init__(self, final_lane, date, assigned_to, name):
        self.final_lane = final_lane
        self.date = date
        self.assigned_to = assigned_to
        self.name = name


class CardAction(object):
    who = u''
    what = u''
    when = u''

    def __init__(self, who, what, when):
        self.who = who
        self.what = what
        self.when = when


def silo_backlog(api, trello_board_id):
    open_cards = _get_open_cards(lambda: _get_all_silo_cards(api, trello_board_id))
    format.print_title("Silo Backlog:")
    format.print_card_table(open_cards)


def silo_stats(api, trello_board_id):
    cards = list(_get_all_completed_cards(api, trello_board_id))
    format.print_last_day_stats(cards)
    format.print_last_week_stats(cards)
    format.print_meta_stats(cards)


def silo_actions(api, trello_board_id):
    actions = _get_all_actions(api, trello_board_id)
    format.print_title("Activity Summary:")
    format.print_actions_table(actions)


def _get_all_completed_cards(trello_api, trello_board_id):
    actions = trello_api.get_boards_action(
        trello_board_id,
        filter='updateCard',
        limit=1000
    )
    for action in actions:
        list_before = action[u'data'].get('listBefore', None)
        list_after = action[u'data'].get('listAfter', None)
        if None in (list_before, list_after):
            # we get events for cards moving position, but not lists.
            continue
        if list_after[u'name'] not in (u'Passed', u'Failed'):
            # Ignore card moves to the 'under test' lane:
            continue

        user_name = action[u'memberCreator'][u'fullName']
        card_name = action[u'data'][u'card'][u'name']
        time_string = action[u'date']

        yield CompletedCard(
            list_after[u'name'],
            dateutil.parser.parse(time_string),
            user_name,
            card_name
        )


def _get_all_actions(trello_api, trello_board_id):
    raw_actions = trello_api.get_boards_action(
        trello_board_id,
        limit=1000
    )
    for action in raw_actions:
        list_before = action[u'data'].get('listBefore', None)
        list_after = action[u'data'].get('listAfter', None)
        user_name = action[u'memberCreator'][u'fullName']
        time_string = action[u'date']
        time_stamp = dateutil.parser.parse(time_string)

        act = action[u'type']

        # skip a few action types
        if act in ('createCard',
                   'deleteCard',
                   'removeMemberFromCard',
                   'updateBoard'):
            continue
        # ignore updates which don't traverse lanes
        # (not entirely sure what all these actions are though)
        if act == 'updateCard' and (None in (list_before, list_after)):
            continue

        a = CardAction(user_name, act, time_stamp)
        yield a


def _get_all_silo_cards(trello_api, trello_board_id):
    """Get a list of cards that are in the 'awaiting QA signoff' lane.

    This function returns a generator that yields a list of Card objects.
    """
    # 'lists' are trello lanes. Build a map of list ids to names:
    # TODO: Make this testable by splitting the trello api calls from the logic
    # in this function.
    list_id_map = {}
    for list_dict in trello_api.get_boards_list(trello_board_id):
        list_id_map[list_dict[u'id']] = list_dict[u'name']

    for card in trello_api.get_boards_card(trello_board_id):
        card_obj = Card()
        card_obj.name = card[u'name']
        card_obj.lane = list_id_map.get(card[u'idList'], u"Unknown")
        yield card_obj


def _get_open_cards(card_generator):
    """Get all the open cards.

    'open', in this context means nobody is actively working on them.

    This function returns a generator that yields Card objects.
    """
    for card in card_generator():
        if card.lane == u'Need QA Sign-off':
            yield card


def add_parsers(subparsers):
    # TODO wonder if this could be improved by sub-parsing under silo, e.g.
    backlog_parser = subparsers.add_parser(
        "silo-backlog",
        help="Show the current silo testing backlog.")
    stats_parser = subparsers.add_parser(
        "silo-stats",
        help="Show statistics around completed silo testing.")
    actions_parser = subparsers.add_parser(
        "silo-actions",
        help="Show statistics about overall activity per person.")
    for parser in [backlog_parser, stats_parser, actions_parser]:
        parser.add_argument('--trello-app-key',
                            required=True,
                            action='store',
                            help='Trello api key.')
        parser.add_argument('--trello-token',
                            required=True,
                            action='store',
                            help='Trello token.')
        parser.add_argument('--trello-board-id',
                            required=True,
                            action='store',
                            help='Trello board ID.')


def main(args):
    # TODO: config parsing, env parsing, etc.
    api = trello.TrelloApi(args.trello_app_key, args.trello_token)
    if args.command == 'silo-backlog':
        silo_backlog(api, args.trello_board_id)
    elif args.command == 'silo-stats':
        silo_stats(api, args.trello_board_id)
    elif args.command == 'silo-actions':
        silo_actions(api, args.trello_board_id)
