# qakit - tools for UEQA engineers
# Copyright (C) 2014 Thomi Richards
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


import datetime

import dateutil
from testtools import TestCase
from testtools.matchers import raises

from qakit import format as _f
from qakit.tests.test_silo import (
    make_passed_completed_card,
    make_failed_completed_card,
)


def get_today_utc():
    return datetime.datetime.utcnow().replace(tzinfo=dateutil.tz.tzutc())


def make_period_with_one_day_duration():
    dt1 = get_today_utc()
    dt2 = dt1 + datetime.timedelta(days=1)
    return _f.Period(dt1, dt2)

class FormatPeriodTests(TestCase):


    def test_can_construct_with_two_dates(self):
        dt1 = get_today_utc()
        dt2 = dt1 + datetime.timedelta(days=1)
        p = _f.Period(dt1, dt2)

    def test_raises_ValueError_if_start_after_end(self):
        dt1 = get_today_utc()
        dt2 = dt1 + datetime.timedelta(days=1)
        self.assertThat(
            lambda: _f.Period(dt2, dt1),
            raises(
                ValueError("Start date must be before or equal to end date")
            )
        )

    def test_can_construct_with_same_date(self):
        dt1 = get_today_utc()
        p = _f.Period(dt1, dt1)

    def test_period_includes_start_date(self):
        p = make_period_with_one_day_duration()
        self.assertTrue(p.contains(p.start))

    def test_period_includes_end_date(self):
        p = make_period_with_one_day_duration()
        self.assertTrue(p.contains(p.end))

    def test_period_excludes_dates_before_start(self):
        p = make_period_with_one_day_duration()
        self.assertFalse(p.contains(p.start - datetime.timedelta(seconds=1)))

    def test_period_excludes_dates_after_end(self):
        p = make_period_with_one_day_duration()
        self.assertFalse(p.contains(p.end + datetime.timedelta(seconds=1)))

    def test_zero_duration_period_contains_start_and_end_dates(self):
        dt1 = get_today_utc()
        p = _f.Period(dt1, dt1)
        self.assertTrue(p.contains(p.start))
        self.assertTrue(p.contains(p.end))

    def test_hashes_are_unique(self):
        p = make_period_with_one_day_duration()
        p2 = make_period_with_one_day_duration()
        p2.end += datetime.timedelta(seconds=1)
        self.assertNotEqual(hash(p), hash(p2))

    def test_hashes_are_equal(self):
        end = get_today_utc()
        start = end - datetime.timedelta(days=1)
        p1 = _f.Period(start, end)
        p2 = _f.Period(start, end)
        self.assertEqual(hash(p1), hash(p2))

    def test_periods_can_be_dictionary_keys(self):
        p = make_period_with_one_day_duration()
        d = {p: "Hello"}
        self.assertEqual("Hello", d[p])

    def test_get_24_hour_period(self):
        p = _f._get_24_hours_ago()
        time_delta = p.end - p.start
        self.assertEqual(
            int(time_delta.total_seconds()),
            seconds_in_24_hours()
        )

    def test_get_week_period(self):
        p = _f._get_one_week_ago()
        time_delta = p.end - p.start
        self.assertEqual(
            int(time_delta.total_seconds()),
            seconds_in_24_hours() * 7
        )

    def test_get_fortnight_period(self):
        p = _f._get_one_fortnight_ago()
        time_delta = p.end - p.start
        self.assertEqual(
            int(time_delta.total_seconds()),
            seconds_in_24_hours() * 14
        )


class PassRateStatisticTests(TestCase):

    def test_sets_name(self):
        s = _f.PassRateStatistic()
        self.assertEqual("Pass Rate", s.name)

    def test_empty_category(self):
        s = _f.PassRateStatistic()
        p = make_period_with_one_day_duration()
        self.assertEqual(s.result_for_period(p), "NA")

    def test_single_passing_card(self):
        s = _f.PassRateStatistic()
        p = make_period_with_one_day_duration()
        c = make_passed_completed_card()
        s.process_card(c, p)
        self.assertEqual(s.result_for_period(p), "100%")

    def test_single_failing_card(self):
        s = _f.PassRateStatistic()
        p = make_period_with_one_day_duration()
        c = make_failed_completed_card()
        s.process_card(c, p)
        self.assertEqual(s.result_for_period(p), "0%")

    def test_multiple_cards(self):
        s = _f.PassRateStatistic()
        p = make_period_with_one_day_duration()
        s.process_card(make_failed_completed_card(), p)
        s.process_card(make_passed_completed_card(), p)
        self.assertEqual(s.result_for_period(p), "50%")


def seconds_in_24_hours():
    return 60 * 60 * 24


def test_suite():
    from unittest import TestLoader
    return TestLoader().loadTestsFromName(__name__)
