

from testtools import TestCase
from testtools.matchers import raises, Equals

import tempfile
from qakit import lkkb
from contextlib import contextmanager


@contextmanager
def temp_config_file(content):
    with tempfile.NamedTemporaryFile('w+') as f:
        f.write(content)
        f.flush()
        f.seek(0)
        yield f.name


class ConfigFileLoadTests(TestCase):

    def test_raises_IOError_if_file_not_found(self):
        name = tempfile.mktemp()
        self.assertThat(
            lambda: lkkb.get_lkkb_auth_pair(name),
            raises(IOError)
        )

    def assert_content_raises_exception(self, content, exception):
        with temp_config_file(content) as name:
            self.assertThat(
                lambda: lkkb.get_lkkb_auth_pair(name),
                raises(exception)
            )

    def test_empty_file_raises_ValueError(self):
        self.assert_content_raises_exception(
            "",
            ValueError("Could not find 'LeanKit' section in config file.")
        )

    def test_empty_section_raises_ValueError(self):
        self.assert_content_raises_exception(
            "[LeanKit]\n",
            ValueError("No username set in config file.")
        )

    def test_empty_user_option_raises_ValueError(self):
        self.assert_content_raises_exception(
            (
                "[LeanKit]\n"
                "Username=\n"
            ),
            ValueError("No username set in config file.")
        )

    def test_missing_password_raises_ValueError(self):
        self.assert_content_raises_exception(
            (
                "[LeanKit]\n"
                "Username=foo\n"
            ),
            ValueError("No password set in config file.")
        )

    def test_empty_password_raises_ValueError(self):
        self.assert_content_raises_exception(
            (
                "[LeanKit]\n"
                "Username=foo\n"
                "Password=\n"
            ),
            ValueError("No password set in config file.")
        )

    def test_can_retrieve_username_and_password(self):
        with temp_config_file(
            "[LeanKit]\n"
            "Username=foo\n"
            "Password=bar\n"
        ) as name:
            self.assertThat(
                lkkb.get_lkkb_auth_pair(name),
                Equals(('foo', 'bar'))
            )


def test_suite():
    from unittest import TestLoader
    return TestLoader().loadTestsFromName(__name__)
